/*      program dirbe_dcaf

c       This procedure will read the DIRBE DCAF FITS
c       Skymaps. It uses the FITSIO library. This program should
c       be called from the IDL procedure DIRBE_READ.PRO
*/

#include "dirbe_inc.h"
#include "dcaf.h" 

static istart,istop;
static IDL_KW_PAR kwpar [3] = {
    { "STARTPIXEL", IDL_TYP_LONG, 1, IDL_KW_ZERO, 0, IDL_CHARA(istart) },
    { "STOPPIXEL", IDL_TYP_LONG, 1, IDL_KW_ZERO, 0, IDL_CHARA(istop) },
    { NULL }
    };

IDL_VARIABLE *dirbe_dcaf(argc,argv,argk)

int argc;
IDL_VARIABLE *argv[];
char *argk;

{
  char comment[73],dataset[255],indxf[255],errtxt[FITS_CLEN_ERRMSG];
  char (*fpt) [FIELDSIZE],fieldname[255],fields [MAXFIELDS] [FIELDSIZE];
  int iunit,status,bitpix,naxis,naxes[99],pcount,gcount;
  int junit,choice,i,j,rwstat,bksize,simple,extend,bandno;
  int hdutyp,inull,n[10],colnum,innum,felem,anyflg;
  int *enums,*nrecs,numargs,nfield,nret,numfield,fflag;
  short *stddev,*psub,*ssof,*stokq,*disp,snull,itemp;
  IDL_LONG *pix,*indpix,dim[2],dim2[2],dim3[2],frow,index,nelmt;
  IDL_LONG startpix,stoppix,startindex,stopindex,startrec,stoprec,numrec;
  float tscale,enull,*solel,*phot,*sigma,*deltat,*stoke,*fracu;
  double dnull,*time;

  IDL_VARIABLE *ret_dat[MAXFIELDS],*posargs[MAXFIELDS+2],*retpt,*dum;

        status=0;
        iunit=15;
        junit=16;
        inull=0;
        enull=0.;
        dnull=0.;
        snull=0;
        frow=1;
        felem=1;
        n[0]=4;
        n[1]=4;
        n[2]=4;
        n[3]=4;
        n[4]=3;
        n[5]=3;
        n[6]=2;
        n[7]=2;
        n[8]=0;
        n[9]=1;
/* define tscale=1/2 bin = 0.5*360/(2**15-1)   */
        tscale=0.00549;

/* define IDL interface stuff                 */
        retpt = (IDL_VARIABLE *) IDL_Gettmp ();
        retpt->type = IDL_TYP_INT;
        numargs = IDL_KWGetParams (argc, argv, argk, kwpar, posargs, 1);
        IDL_KWCleanup (IDL_KW_CLEAN_ALL);
        if (posargs[0]->flags & IDL_V_NOT_SCALAR ||
             posargs[0]->type != IDL_TYP_STRING)
             IDL_MessageErrno (IDL_M_NAMED_GENERIC,0,IDL_MSG_LONGJMP,
                "Invalid 'dataset' argument data type/class.");
        if (posargs[0]->value.str.slen == 0)
        {
           printf ("Enter File Name: ");
           scanf ("%s",dataset);
        } 
        else 
           strcpy (dataset, posargs[0]->value.str.s);
        if (posargs[1]->flags & IDL_V_NOT_SCALAR ||
            posargs[1]->type != IDL_TYP_STRING)
            IDL_MessageErrno (IDL_M_NAMED_GENERIC,0,IDL_MSG_LONGJMP,
             "Invalid 'field' argument data type/class.");
        if (posargs[1]->value.str.slen == 0)
           {
           printf ("This file contains the following fields:\n");
           for (index=0; index < MAXFIELDS; index++)
              printf (" %s \n",ttype[index].name);
           printf ("Enter Field Name as shown : ");
           scanf ("%s",fieldname);
           }
        else 
           strcpy (fieldname, posargs[1]->value.str.s);
        fpt = fields;  fpt++;
        nfield = getfields (MAXFIELDS, fieldname, fpt);
        if (nfield < 0)
          IDL_MessageErrno (IDL_M_NAMED_GENERIC,0,IDL_MSG_LONGJMP,
             "Only up to 15 fields may be specified.");
        nret = numargs - 2;
        if (nret < nfield)
          IDL_MessageErrno (IDL_M_NAMED_GENERIC, 0, IDL_MSG_LONGJMP,
          "There are not enough IDL IDL_VARIABLEs to return the fields requested.");
        nfield +=1;

/*C     open the existing FITS file with readonly access  */
        rwstat=0;
        FCOPEN(iunit, dataset, rwstat, &bksize, &status);
        FCGERR(status,errtxt);

/*C     read the required primary array keywords  */
        FCGHPR(iunit,99,&simple,&bitpix,&naxis,naxes,&pcount,
               &gcount,&extend,&status);
        FCGERR(status,errtxt);

/*      getting pixel index file */

        printf ("Reading Pixel Index file\n");
        FCOPEN(junit,getix(dataset),rwstat, &bksize, &status);
        FCGERR(status,errtxt);
        FCMAHD(junit,2,&hdutyp,&status);
        FCGERR(status,errtxt);
        FCGKYJ(junit,"NAXIS2",&nelmt,comment, &status);
        dim[0]=nelmt;
        innum=1;
        indpix = (IDL_LONG *) IDL_MakeTempArray(IDL_TYP_LONG,1,dim,
                   IDL_BARR_INI_ZERO,&dum);
        FCGCVJ(junit,innum,frow,felem,nelmt,inull,indpix,&anyflg,&status);
        FCGERR(status,errtxt);
        if (istart == 0 )
        {
           printf ("Enter Start pixel number: ");
           scanf ("%d",&startpix);
        }
        else 
           startpix = istart;
        if (istop == 0 )
        {
           printf ("Enter Last pixel number: ");
           scanf ("%d",&stoppix);
        }
        else 
           stoppix = istop;
        for (index=0; index < nelmt; index++)
            if (indpix[index] == startpix) break;
        startindex=index;
        for (index=startindex; index < nelmt; index++)
            if (indpix[index] == stoppix) break;
        stopindex=index;
        innum=2;
        nelmt=1;
        frow=startindex+1l;
        FCGCVJ(junit,innum,frow,felem,nelmt,inull,&startrec,&anyflg,&status);
        FCGERR(status,errtxt);
        frow=stopindex+2l;
        FCGCVJ(junit,innum,frow,felem,nelmt,inull,&stoprec,&anyflg,&status);
        FCGERR(status,errtxt);
        numrec=stoprec-startrec;
        IDL_DELTMP(dum);
        FCCLOS(junit,&status);
        FCGERR(status,errtxt);
/*      end index file         */

/*C     now move to the next ASCII table extension */
        FCMAHD(iunit,2,&hdutyp,&status);
        FCGERR(status,errtxt);
        printf ("Reading Data file\n");

        for (numargs=0; numargs < nfield-1; numargs++)
        {
        fflag=0;
        numfield=0;
        strcpy(dataset,ttype[numfield].name);
        while (strcmp(strlwr(*fpt),strlwr(dataset)) && numfield < MAXFIELDS)
            {
            ++numfield;
            if (numfield != MAXFIELDS) strcpy(dataset,ttype[numfield].name);
            }
        if (numfield == MAXFIELDS) 
           {
            colnum=99;
            fflag=1;
            printf("Entered an Illegal Field Name %s \n",fpt);
           }
        else 
           colnum=ttype[numfield].colnum;

        frow=startrec;
        nelmt=numrec;
        dim[0]=nelmt;
        dim2[0]=10;
        dim2[1]=nelmt;
        dim3[0]=6;
        dim3[1]=nelmt;

        switch (colnum) {
          case 1:
            pix = (IDL_LONG *) IDL_MakeTempArray(IDL_TYP_LONG,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVJ(iunit,colnum,frow,felem,dim[0],inull,pix,&anyflg,&status);
            FCGERR(status,errtxt);
            break;
          case 2:
            psub = (short *) IDL_MakeTempArray(IDL_TYP_INT,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVI(iunit,colnum,frow,felem,dim[0],snull,psub,&anyflg,&status);
            FCGERR(status,errtxt);
            break;
          case 3:
            dim2[0]=4;
            disp = (short *) IDL_MakeTempArray(IDL_TYP_INT,2,dim2,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVI(iunit,colnum,frow,felem,dim2[0]*dim2[1],snull,disp,&anyflg,&status);
            FCGERR(status,errtxt);
            break;
          case 4:
            time = (double *) IDL_MakeTempArray(IDL_TYP_DOUBLE,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVD(iunit,colnum,frow,felem,dim[0],dnull,time,&anyflg,&status);
            FCGERR(status,errtxt);
            break;
          case 5:
            deltat = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,2,dim2,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVE(iunit,colnum,frow,felem,dim2[0]*dim2[1],enull,deltat,&anyflg,&status);
            FCGERR(status,errtxt);
            for (frow = 0; frow < dim2[0]*dim2[1] ; frow++)
                {
                 if (deltat[frow] < 128*4725.) 
                    deltat[frow] += 0.5*4725.;
                 else 
                    deltat[frow] -= 255.5*4725.;
                }
            break;
          case 6:
            solel = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVE(iunit,colnum,frow,felem,dim[0],enull,solel,&anyflg,&status);
            FCGERR(status,errtxt);
            for (frow = 0; frow < dim[0] ; frow++)
                solel[frow] += tscale;
            break;
          case 7:
            phot = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,2,dim2,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVE(iunit,colnum,frow,felem,dim2[0]*dim2[1],enull,phot,&anyflg,&status);
            FCGERR(status,errtxt);
            break;
          case 8:
            stoke = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,2,dim3,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVE(iunit,colnum,frow,felem,dim3[0]*dim3[1],enull,stoke,&anyflg,&status);
            FCGERR(status,errtxt);               
             break;
          case 9:
            nrecs = (int *) IDL_MakeTempArray(IDL_TYP_INT,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVI(iunit,colnum,frow,felem,dim[0],inull,nrecs,&anyflg,&status);
            FCGERR(status,errtxt);
            break;
          case 10:
            enums = (int *) IDL_MakeTempArray(IDL_TYP_INT,2,dim2,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVI(iunit,colnum,frow,felem,dim2[0]*dim2[1],inull,enums,&anyflg,&status);
            FCGERR(status,errtxt);
            break;
          case 11:
            stddev = (short *) IDL_MakeTempArray(IDL_TYP_INT,2,dim2,
                       IDL_BARR_INI_ZERO,&dum);
            FCGCVI(iunit,colnum,frow,felem,dim2[0]*dim2[1],snull,stddev,&anyflg,&status);
            FCGERR(status,errtxt);
            sigma = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,2,dim2,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            for (frow = 0; frow < dim2[1] ; frow++)
              {
              for (bandno = 0; bandno < dim2[0]; bandno++)
                 {
                 index=bandno + frow*dim2[0];
                 itemp = *(stddev+index);
                 if (itemp == 0)
                    *(sigma+index) = pow(10.,0.-n[bandno]);
                 else if (itemp == 255)
                    *(sigma+index) = pow(10.,4-n[bandno]);
                 else
                    *(sigma+index) = pow(10.,(itemp-0.5)/63.5-n[bandno]);
                 }
              } 
            IDL_DELTMP(dum);
            break;
          case 12:
            stddev = (short *) IDL_MakeTempArray(IDL_TYP_INT,2,dim3,
                       IDL_BARR_INI_ZERO,&dum);
            FCGCVI(iunit,colnum,frow,felem,dim3[0]*dim3[1],snull,stddev,&anyflg,&status);
            FCGERR(status,errtxt);
            sigma = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,2,dim3,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            for (frow = 0; frow < dim3[0]*dim3[1] ; frow++)
              {
              itemp = *(stddev+frow);
              if (itemp == 0)
                 *(sigma+frow) = 0.0001;
              else if (itemp == 255)
                 *(sigma+frow) = 1.;
              else
                 *(sigma+frow) = pow(10.,(itemp-0.5)/63.5-4);
              } 
            IDL_DELTMP(dum);
            break;
          case 13:
            ssof = (short *) IDL_MakeTempArray(IDL_TYP_INT,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVI(iunit,colnum,frow,felem,dim[0],snull,ssof,&anyflg,&status);
            FCGERR(status,errtxt);
            break;
          case 14:
            dim2[0]=3;
            fracu = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,2,dim2,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVE(iunit,colnum,frow,felem,dim2[0]*dim2[1],enull,fracu,&anyflg,&status);
            FCGERR(status,errtxt);
            break;
          case 15:
            stokq = (short *) IDL_MakeTempArray(IDL_TYP_INT,2,dim3,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVI(iunit,colnum,frow,felem,dim3[0]*dim3[1],snull,stokq,&anyflg,&status);
            FCGERR(status,errtxt);
            break;
          default:
            break;
        }
        fpt++;
        if (status <= 0 && fflag ==0)
            IDL_VarCopy(ret_dat[numargs],argv[numargs+2]);
        }
/*C     now close the table and quit */
        FCCLOS(iunit,&status);
        FCGERR(status,errtxt);

  if (status <= 0 && fflag ==0)
    {
      retpt->value.i=1;
      puts ("*** Program completed successfully ***");
    }
  else
    {

    /*C      get the error text description */
      FCGERR(status,errtxt);
      puts ("*** ERROR - program did not run successfully ***");
      printf ("status = %d : %s\n", status, errtxt);
     }
return retpt;
}
