/*      program dirbe_gpm

c       This procedure will read the DIRBE Galactic Plane FITS
c       Skymaps. It uses the FITSIO library. This program should
c       be called from the IDL procedure DIRBE_READ.PRO
*/

#include "dirbe_inc.h"
#include "gpm.h" 

static IDL_KW_PAR kwpar [1] = {
    { NULL }
    };

IDL_VARIABLE *dirbe_gpm(argc,argv,argk)

int argc;
IDL_VARIABLE *argv[];
char *argk;

{
  char dataset[255],version[73];
  char (*fpt) [FIELDSIZE],fieldname[255],fields [MAXFIELDS] [FIELDSIZE];
  int *enums,*nrecs,numargs,nfield,nret,numfield,fflag;
  char comment[73],errtxt[FITS_CLEN_ERRMSG];
  int iunit,status,bitpix,naxis,naxes[99],pcount,gcount;
  int choice,i,j,rwstat,bksize;
  int hdutyp,inull,n,colnum,felem;
  int simple,extend,anyflg,bandno;
  short *stddev,snull;
  unsigned int itemp,*pqual;
  IDL_LONG *pix,dim[2],dim2[2],frow,index,nelmt;
  float tscale,enull,*phot,*sigma;
  float *elon,*elat,*glon,*glat;
  double dnull,*time;


  IDL_VARIABLE *ret_dat[MAXFIELDS],*posargs[MAXFIELDS+2],*retpt,*dum,*dumer;

        status=0;
        iunit=15;
        inull=0;
        enull=0.;
        dnull=0.;
        snull=0;
        frow=1;
        felem=1;
/* define tscale=1/2 bin = 0.5*360/(2**15-1)   */
        tscale=0.0054933;

/* define IDL interface stuff                 */
        retpt = (IDL_VARIABLE *) IDL_Gettmp ();
        retpt->type = IDL_TYP_INT;
        numargs = IDL_KWGetParams (argc, argv, argk, kwpar, posargs, 1);
        IDL_KWCleanup (IDL_KW_CLEAN_ALL);
        if (posargs[0]->flags & IDL_V_NOT_SCALAR ||
             posargs[0]->type != IDL_TYP_STRING)
             IDL_MessageErrno (IDL_M_NAMED_GENERIC,0,IDL_MSG_LONGJMP,
                "Invalid 'dataset' argument data type/class.");
        if (posargs[0]->value.str.slen == 0)
        {
           printf ("Enter File Name: ");
           scanf ("%s",dataset);
        } 
        else 
           strcpy (dataset, posargs[0]->value.str.s);
        if (posargs[1]->flags & IDL_V_NOT_SCALAR ||
            posargs[1]->type != IDL_TYP_STRING)
            IDL_MessageErrno (IDL_M_NAMED_GENERIC,0,IDL_MSG_LONGJMP,
             "Invalid 'field' argument data type/class.");
        if (posargs[1]->value.str.slen == 0)
           {
           printf ("This file contains the following fields:\n");
           for (index=0; index < MAXFIELDS; index++)
              printf (" %s \n",ttype[index].name);
           printf ("Enter Field Name as shown : ");
           scanf ("%s",fieldname);
           }
        else 
           strcpy (fieldname, posargs[1]->value.str.s);
        fpt = fields;  fpt++;
        nfield = getfields (MAXFIELDS, fieldname, fpt);
        if (nfield < 0)
          IDL_MessageErrno (IDL_M_NAMED_GENERIC,0,IDL_MSG_LONGJMP,
             "Only up to 8 fields may be specified.");
        nret = numargs - 2;
        if (nret < nfield)
          IDL_MessageErrno (IDL_M_NAMED_GENERIC, 0, IDL_MSG_LONGJMP,
          "There are not enough IDL IDL_VARIABLEs to return the fields requested.");
        nfield +=1;

        printf ("Reading Data file\n");

/*C     open the existing FITS file with readonly access  */
        rwstat=0;
        FCOPEN(iunit, dataset, rwstat, &bksize, &status);
        FCGERR(status,errtxt);

/*C     read the required primary array keywords  */
        FCGHPR(iunit,99,&simple,&bitpix,&naxis,naxes,&pcount,
               &gcount,&extend,&status);
        FCGERR(status,errtxt);

/*C     test for processing version        */
        FCGKYS(iunit,"VERSION",version,comment, &status);
        FCGERR(status,errtxt);
                       
/*C     now move to the next ASCII table extension */
        FCMAHD(iunit,2,&hdutyp,&status);
        FCGERR(status,errtxt);
 
/*C     read a naxis2 and set row dimensions */
        FCGKYJ(iunit,"NAXIS2",&nelmt,comment, &status);
        dim[0]=nelmt;
        dim2[0]=10;
        dim2[1]=nelmt;

        for (numargs=0; numargs < nfield-1; numargs++)
        {
        fflag=0;
        numfield=0;
        strcpy(dataset,ttype[numfield].name);
        while (strcmp(strlwr(*fpt),strlwr(dataset)) && numfield < MAXFIELDS)
            {
            ++numfield;
            if (numfield != MAXFIELDS) strcpy(dataset,ttype[numfield].name);
            }
        if (numfield == MAXFIELDS) 
           {
            colnum=99;
            fflag=1;
            printf("Entered an Illegal Field Name %s \n",fpt);
           }
        else 
           colnum=ttype[numfield].colnum;
        
        switch (colnum) {
          case 1:
            pix = (IDL_LONG *) IDL_MakeTempArray(IDL_TYP_LONG,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVJ(iunit,colnum,frow,felem,dim[0],inull,pix,&anyflg,&status);
            FCGERR(status,errtxt);
            break;
          case 2:
            phot = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,2,dim2,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVE(iunit,colnum,frow,felem,dim2[0]*dim2[1],enull,phot,&anyflg,&status);
            FCGERR(status,errtxt);
            break;
          case 3:
            stddev = (short *) IDL_MakeTempArray(IDL_TYP_INT,2,dim2,
                       IDL_BARR_INI_ZERO,&dum);
            FCGCVI(iunit,colnum,frow,felem,dim2[0]*dim2[1],snull,stddev,&anyflg,&status);
            FCGERR(status,errtxt);
            printf ("You can choose either: \n");
            printf ("   Standard Deviation (1) \n");
            printf ("   Quality flag (2) \n");
            printf ("Enter Choice (1 or 2): ");
            scanf ("%d",&choice);
            if (choice == 2) 
                {
                pqual = (unsigned int *) IDL_MakeTempArray(IDL_TYP_LONG,2,dim2,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
                for (frow = 0; frow < dim2[0]*dim2[1]; frow++)
                   {
                   itemp = *(stddev+frow)/256;
                   *(pqual+frow) = *(stddev+frow) - itemp*256;
                   }
                IDL_DELTMP(dum);
                break;
                }
            else
            {
            sigma = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,2,dim2,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            colnum=2;
            phot = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,2,dim2,
                    IDL_BARR_INI_ZERO,&dumer);
            FCGCVE(iunit,colnum,frow,felem,dim2[0]*dim2[1],enull,phot,&anyflg,&status);
            FCGERR(status,errtxt);
            for (bandno = 0; bandno < dim2[0]; bandno++)
              {
              n=3;
              if (bandno >= 8) 
                  n=1;
              if (bandno == 4) 
                  n=4;
              if (bandno == 5) 
                  n=4;
              for (frow = 0; frow < dim2[1] ; frow++)
                 {
                 index=bandno + frow*dim2[0];
                 itemp = *(stddev+index)/256;
                 if (itemp == 0)
                    *(sigma+index) = pow(10.,0.-n);
                 else if (itemp == 255)
                    *(sigma+index) = pow(10.,4-n);
                 else
                    *(sigma+index) = pow(10.,(itemp-0.5)/63.5-n);
                 if (bandno < 8)
                     *(sigma+index) *= fabs(*(phot+index));
                 }
              } 
            IDL_DELTMP(dum);
            IDL_DELTMP(dumer);
            break;
            }
          case 4:
            time = (double *) IDL_MakeTempArray(IDL_TYP_DOUBLE,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVD(iunit,colnum,frow,felem,dim[0],dnull,time,&anyflg,&status);
            FCGERR(status,errtxt);
            break;
          case 5:
            elon = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVE(iunit,colnum,frow,felem,dim[0],enull,elon,&anyflg,&status);

            FCGERR(status,errtxt);
            if (strcmp(version,"Pass 2B") == 0) 
               {
               for (frow = 0; frow < nelmt; frow++)
                 {
                 if (elon[frow] >= tscale*2.)
                    elon[frow] += tscale;
                 if (elon[frow] <= -(tscale*2))
                    elon[frow] -= tscale;
                 }
               }
            break;
          case 6:
            elat = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVE(iunit,colnum,frow,felem,dim[0],enull,elat,&anyflg,&status);
            FCGERR(status,errtxt);
            if (strcmp(version,"Pass 2B") == 0) 
               {
               for (frow = 0; frow < nelmt; frow++)
                 {
                 if (elat[frow] >= tscale*2.)
                    elat[frow]+=tscale;
                 if (elat[frow] <= -(tscale*2))
                    elat[frow] -= tscale;
                 }
               }
            break;
          case 7:
            glon = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVE(iunit,colnum,frow,felem,dim[0],enull,glon,&anyflg,&status);
            FCGERR(status,errtxt);
            if (strcmp(version,"Pass 2B") == 0) 
               {
               for (frow = 0; frow < nelmt; frow++)
                 {
                 if (glon[frow] >= tscale*2.)
                    glon[frow] += tscale;
                 if (glon[frow] <= -(tscale*2))
                    glon[frow] -= tscale;
                 }
               }
            break;
          case 8:
            glat = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            FCGCVE(iunit,colnum,frow,felem,dim[0],enull,glat,&anyflg,&status);
            FCGERR(status,errtxt);
            if (strcmp(version,"Pass 2B") == 0) 
               {
               for (frow = 0; frow < nelmt; frow++)
                 {
                 if (glat[frow] >= tscale*2.)
                    glat[frow]+=tscale;
                 if (glat[frow] <= -(tscale*2))
                    glat[frow] -= tscale;
                 }
               }
            break;
          default:
            break;
        }

        fpt++;
        if (status <= 0 && fflag ==0)
            IDL_VarCopy(ret_dat[numargs],argv[numargs+2]);
        }
/*C     now close the table and quit */
        FCCLOS(iunit,&status);
        FCGERR(status,errtxt);

  if (status <= 0 && fflag ==0)
    {
      retpt->value.i=1;
      puts ("*** Program completed successfully ***");
    }
  else
    {

    /*C      get the error text description */
      FCGERR(status,errtxt);
      puts ("*** ERROR - program did not run successfully ***");
      printf ("status = %d : %s\n", status, errtxt);
     }
return retpt;
}
