/*******************************************************************************
   MODULE:	getfields

   FUNCTION:	This routine parses the input fieldname string and returns 
		individual fields.

   AUTHOR:	S. Masiee, STX, 2/91 (SER# 9311) - Modify Dataserver routines to
						   handle multiple fields.
   MODIFICATION HISTORY:
	DATE	SPR #	VERS	PROGRAMMER	COMMENTS
	04/92	9687		S. Masiee	Port back to Unix linking to IDL
	07/93	11155	10.9	S.Masiee, HSTX	Modify to compile under Vax-C

   INPUT PARAMETERS:	maxfld	   int		Max number of fields permitted.
						  Dataserver's maximum is 
						  initially set to 8 + pixel_no.
			fieldin	   char pt	Ptr to input fieldname string

   OUTPUT PARAMETERS:	fpt	   pt to array	Array of the fields requested
				    of strings

   RETURN:		nfield	   int		Number of fields in input string

*******************************************************************************/

#include <stdio.h>
#include <string.h>
#define FIELDSIZE 255

int getfields (int maxfld, char *fieldin, char (*fpt) [FIELDSIZE] )
{
  int	    nfield;
  char	    *found, *fnext, *fparen, *fpend;

/* Retrieve the fieldname(s) from the input parameter. */

  strcat (fieldin, ",");
  found = fieldin;
  fnext = (char *) strchr (found, ',');
  fparen = (char *) strchr (found, '(');

  for (nfield=0;  fnext != 0;  fpt++) {
    if (fparen != 0 && fparen < fnext) {
	fpend = (char *) strchr (fparen, ')');
	if (fpend != 0)
	  fnext = (char *) strchr (fpend, ',');
	fparen = (char *) strchr (fnext, '(');
    }
    nfield++;

/* Copy each field into a separate string variable. 
   If too many fields, return, sending flag. */

    if (nfield > maxfld) {
	nfield = -1;
	break;
    }
    *fnext = '\0';
    strcpy (*fpt, found);

    found = fnext + 1;
    fnext = (char *) strchr (found,',');
  }

  return nfield;
}
