/*      program dirbe_aaf

c       This procedure will read the DIRBE Annual Averaged FITS
c       Skymaps. It uses the FITSIO library. This program should
c       be called from the IDL procedure DIRBE_READ.PRO

*/

#include "dirbe_inc.h"
#include "aaf.h" 

static IDL_KW_PAR kwpar [1] = {
    { NULL }
    };

IDL_VARIABLE *dirbe_aaf(argc,argv,argk)

int argc;
IDL_VARIABLE *argv[];
char *argk;

{
  char dataset[255];
  char (*fpt) [FIELDSIZE],fieldname[255],fields [MAXFIELDS] [FIELDSIZE];
  char comment[73],errtxt[FLEN_ERRMSG];
  int *enums,*nrecs,numargs,nfield,nret,numfield,fflag;
  int status,bitpix,naxis,knull;
  int i,j,hdutyp,n,colnum,felem,simple,extend,anyflg;
  short *psub,snull;
  IDL_LONG *pix,*srecs,dim[2];
  long inull,frow,index,naxes[99],pcount,gcount,nelmt;
  float enull,*phot,*sigma;
  double dnull,*time;
  fitsfile *iunit;

  IDL_VARIABLE *ret_dat[MAXFIELDS],*posargs[MAXFIELDS+2],*retpt,*dum;

        status=0;
        inull=0;
        snull=0;
        enull=0.;
        dnull=0.;
        frow=1;
        felem=1;

/* define IDL interface stuff                 */
        retpt = (IDL_VARIABLE *) IDL_Gettmp ();
        retpt->type = IDL_TYP_INT;
        numargs = IDL_KWGetParams (argc, argv, argk, kwpar, posargs, 1);
        IDL_KWCleanup (IDL_KW_CLEAN_ALL);
        if (posargs[0]->flags & IDL_V_NOT_SCALAR ||
             posargs[0]->type != IDL_TYP_STRING)
             IDL_MessageErrno (IDL_M_NAMED_GENERIC,0,IDL_MSG_LONGJMP,
                "Invalid 'dataset' argument data type/class.");
        if (posargs[0]->value.str.slen == 0)
        {
           printf ("Enter File Name: ");
           scanf ("%s",dataset);
        } 
        else 
           strcpy (dataset, posargs[0]->value.str.s);
        if (posargs[1]->flags & IDL_V_NOT_SCALAR ||
            posargs[1]->type != IDL_TYP_STRING)
            IDL_MessageErrno (IDL_M_NAMED_GENERIC,0,IDL_MSG_LONGJMP,
             "Invalid 'field' argument data type/class.");
        if (posargs[1]->value.str.slen == 0)
           {
           printf ("This file contains the following fields:\n");
           for (index=0; index < MAXFIELDS; index++)
              printf (" %s \n",ttype[index].name);
           printf ("Enter Field Name as shown : ");
           scanf ("%s",fieldname);
           }
        else 
           strcpy (fieldname, posargs[1]->value.str.s);
        fpt = fields;  fpt++;
        nfield = getfields (MAXFIELDS, fieldname, fpt);
        if (nfield < 0)
          IDL_MessageErrno (IDL_M_NAMED_GENERIC,0,IDL_MSG_LONGJMP,
             "Only up to 7 fields may be specified.");
        nret = numargs - 2;
        if (nret < nfield)
          IDL_MessageErrno (IDL_M_NAMED_GENERIC, 0, IDL_MSG_LONGJMP,
          "There are not enough IDL IDL_VARIABLEs to return the fields requested.");
        nfield +=1;

        printf ("Reading Data file\n");

/*C     open the existing FITS file with readonly access  */
        fits_open_file(&iunit, dataset, READONLY, &status);
        fits_get_errstatus(status,errtxt);

/*C     read the required primary array keywords  */
        fits_read_imghdr(iunit,99,&simple,&bitpix,&naxis,naxes,&pcount,
               &gcount,&extend,&status);
        fits_get_errstatus(status,errtxt);

/*C     now move to the next ASCII table extension */
        fits_movabs_hdu(iunit,2,&hdutyp,&status);
        fits_get_errstatus(status,errtxt);
/*C     read a naxis2 and set row dimensions */
        fits_read_key_lng(iunit,"NAXIS2",&nelmt,comment,&status);
        dim[0]=nelmt;
        for (numargs=0; numargs < nfield-1; numargs++)
        {
        fflag=0;
        numfield=0;
        strcpy(dataset,ttype[numfield].name);
        while (strcmp(strlwr(*fpt),strlwr(dataset)) && numfield < MAXFIELDS)
            {
            ++numfield;
            if (numfield != MAXFIELDS) strcpy(dataset,ttype[numfield].name);
            }
        if (numfield == MAXFIELDS) 
           {
            colnum=99;
            fflag=1;
            printf("Entered an Illegal Field Name %s \n",fpt);
           }
        else 
           colnum=ttype[numfield].colnum;

        switch (colnum) {
          case 1:
            pix = (IDL_LONG *) IDL_MakeTempArray(IDL_TYP_LONG,1,dim, 
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            ffgcvk(iunit,colnum,frow,felem,dim[0],inull,pix,&anyflg,&status);
            fits_get_errstatus(status,errtxt);
            break;
          case 2:
            psub = (short *) IDL_MakeTempArray(IDL_TYP_INT,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            ffgcvi(iunit,colnum,frow,felem,dim[0],snull,psub,&anyflg,&status);
            fits_get_errstatus(status,errtxt);
            break;
          case 3:
            time = (double *) IDL_MakeTempArray(IDL_TYP_DOUBLE,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            ffgcvd(iunit,colnum,frow,felem,dim[0],dnull,time,&anyflg,&status);
            fits_get_errstatus(status,errtxt);
            break;
          case 4:
            phot = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            ffgcve(iunit,colnum,frow,felem,dim[0],enull,phot,&anyflg,&status);
            fits_get_errstatus(status,errtxt);
            break;
          case 5:
            sigma = (float *) IDL_MakeTempArray(IDL_TYP_FLOAT,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            ffgcve(iunit,colnum,frow,felem,dim[0],enull,sigma,&anyflg,&status);
            fits_get_errstatus(status,errtxt);
            break;
          case 6:
            enums = (int *) IDL_MakeTempArray(IDL_TYP_LONG,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            ffgcvk(iunit,colnum,frow,felem,dim[0],knull,enums,&anyflg,&status);
            fits_get_errstatus(status,errtxt);
            break;
          case 7:
            srecs = (IDL_LONG *) IDL_MakeTempArray(IDL_TYP_LONG,1,dim,
                       IDL_BARR_INI_ZERO,&ret_dat[numargs]);
            ffgcvk(iunit,colnum,frow,felem,dim[0],inull,srecs,&anyflg,&status);
            fits_get_errstatus(status,errtxt);
            break;
          default:
            break;
        }

        fpt++;
        if (status <= 0 && fflag ==0)
            IDL_VarCopy(ret_dat[numargs],argv[numargs+2]);
        }
/*C     now close the table and quit */
        fits_close_file(iunit,&status);
        fits_get_errstatus(status,errtxt);

  if (status <= 0 && fflag ==0)
    {
      retpt->value.i=1;
      puts ("*** Program completed successfully ***");
    }
  else
    {

    /*C      get the error text description */
      fits_get_errstatus(status,errtxt);
      puts ("*** ERROR - program did not run successfully ***");
      printf ("status = %d : %s\n", status, errtxt);
     }
return retpt;
}
